/*_############################################################################
  _## 
  _##  SNMP4J-Model - SnmpProxyFactory.java  
  _## 
  _##  Copyright (C) 2013-2022  Frank Fock (SNMP4J.org)
  _##  
  _##  This program is free software; you can redistribute it and/or modify
  _##  it under the terms of the GNU General Public License version 2 as 
  _##  published by the Free Software Foundation.
  _##
  _##  This program is distributed in the hope that it will be useful,
  _##  but WITHOUT ANY WARRANTY; without even the implied warranty of
  _##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  _##  GNU General Public License for more details.
  _##
  _##  You should have received a copy of the GNU General Public License
  _##  along with this program; if not, write to the Free Software
  _##  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
  _##  MA  02110-1301  USA
  _##  
  _##########################################################################*/
package org.snmp4j.model.snmp.spi;

import org.snmp4j.model.snmp.proxy.*;
import org.snmp4j.model.snmp.proxy.impl.SnmpColumn;
import org.snmp4j.model.snmp.proxy.impl.SnmpIndexColumn;
import org.snmp4j.smi.OID;
import org.snmp4j.smi.OctetString;

import java.util.List;

/**
 * The {@link SnmpProxyFactory} creates {@link org.snmp4j.model.snmp.proxy.SnmpProxyObject}s.
 *
 * @param <T> the value representation (base) type, i.e. {@link java.lang.String} if you use a
 * {@link org.snmp4j.model.mapper.VariableBindingMapper} that provides {@link java.lang.String} instances for all
 * Snmp types.
 * @param <IC>
 *   the index proxy column type for the table proxy.
 * @param <C>
 *   the proxy column type for the table proxy.
 * @param <IT>
 *   the model value type for index columns.
 * @param <R>
 *   the proxy row type for the rows of the table.
 *
 * @author Frank Fock
 * @version 1.0
 */
public interface SnmpProxyFactory<T,C extends SnmpProxyColumn<T>,IC extends SnmpProxyIndexColumn<T>,
        IT,R extends SnmpProxyRow<IT, T>> {

    SnmpScalar<T> createSnmpScalar(OID oid, OctetString context, SnmpProxyCommitter snmpProxyCommitter,
                                   T value, Class<T> valueClass, int snmpSyntax);

    SnmpTable<C,IC,IT,T,R> createSnmpTable(OID oid, OctetString context, SnmpProxyCommitter snmpProxyCommitter,
                                           SnmpProxyRowFactory<R, IT, T> rowFactory,
                                           List<IC> indexColumnList,
                                           List<C> columnList, int pageSize);
}
